; Copyright (c) 2004-2010, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   PRO osiris_calibrate_dir, in_dir, out_dir_level2, out_dir_level3
;
; PURPOSE:
;   Calibrate a fill directory of images
;
; CATEGORY:
;   PRO osiris_calibrate_dir, in_dir, out_dir_level2, out_dir_level3
;
; CALLING SEQUENCE:
;  osiris_calibrate_dir, in_dir, out_dir_level2, out_dir_level3
;
; INPUTS:
;   in_dir:			The directory to be calibrated
;					If only in_dir is specified (not any of the out_dir_xxx inputs then in_dir is expected to point to the root of an OSIRIS
;					archive node. The procedure will then actually read the input files from <in_dir>/pds/level1 and write the output to
;					out_dir_level2 = <in_dir>/pds/level2    and     out_dir_level3 = <in_dir>/pds/level3
;
; OPTIONAL INPUT:
;   out_dir_level2:	The output directory for OSIRIS level 2 (CODMAC level 3) data
;   out_dir_level3:	The output directory for OSIRIS level 3 (CODMAC level 4) data
;
; KEYWORD PARAMETERS:
;   REPROCESS=REPROCESS:			Force a reprocessing of all the image (if not set only image that have not yet been processed will be used)
;	NAC=NAC:					 	If set only NAC images will be processed
;	WAC=WAC:						If set only WAC images will be processed
;	FILTER=FILTER,:					If set to wildcard string the FILTEr will be used as a search mask for theinput selection
;	LOGFILE=LOGFILE:				The filename of the output processing log
;	DELETE_HISTORY=DELETE_HISTORY:	If set the processing history will be deleted at the end of processing
;	PDS_ARCHIVE=PDS_ARCHIVE:		If set the processing run is for a PDS/PSA archive generation and PDS compatibility will be ensured
;   SKIP_LEVEL3						If set the processing will skip the level three calibration step
;	JPEG_LEVEL2_DIR:				Specify a path to where level2 jpeg files should be written
;	THUMB_LEVEL2_DIR:				Specify a path to where level2 thumbnail jpeg files should be written
;	JPEG_LEVEL3_DIR:					Specify a path to where level3 jpeg files should be written
;	THUMB_LEVEL2_DIR:				Specify a path to where level2 thumbnail jpeg files should be written
;
; OUTPUTS:
;   Nothing
;
; RETURNS:
;   Nothing
;
; RESTRICTIONS:
;   function requires that the paths defined in get_osiris_environment.pro has been set to correctly
;   point to the root directory of the external calibration files
;
; EXAMPLE:
;   osiris_calibrate_dir, 'c:/temp/session_archive/test_session'		; calibrate using the std archive structure
;   osiris_calibrate_dir, 'c:/dir_in', 'c:/level2out', 'c:/level3out'	; calibrate using specific paths
;   osiris_calibrate_dir, 'c:/dir_in', /PDS_ARCHIVE, /SKIP_LEVEL3		; to calibrate a PDS archive node
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid   28/6-2005
;                Sfh 02102006 - Switched to using osiris_model for detecting the camera and hardware model
;-
PRO osiris_calibrate_dir, in_dir, out_dir_level2, out_dir_level3, $
                          REPROCESS=REPROCESS, $
                          NAC=NAC, WAC=WAC, $
                          FILTER=FILTER, $
                          LOGFILE=LOGFILE, $
                          DELETE_HISTORY=DELETE_HISTORY, $
                          PDS_ARCHIVE=PDS_ARCHIVE, $
                          SKIP_LEVEL3=SKIP_LEVEL3, $
                          JPEG_LEVEL2_DIR=JPEG_LEVEL2_DIR, $
						  THUMB_LEVEL2_DIR=THUMB_LEVEL2_DIR, $
						  JPEG_LEVEL3_DIR=JPEG_LEVEL3_DIR, $
						  THUMB_LEVEL3_DIR=THUMB_LEVEL3_DIR, $
                          _EXTRA=_EXTRA

    if n_elements(in_dir) eq 0 then return

    if n_elements(out_dir_level2) eq 0 then begin
        fs = p_parse_file_path(in_dir)
        if fs.name eq 'level1' then in_dir = fs.dir
        fs = p_parse_file_path(in_dir)
        if fs.name eq 'pds' then in_dir = fs.dir

        session_dir = in_dir

        in_dir = in_dir + '\pds\level1'
    endif

    if n_elements(out_dir_level2) eq 0 then begin
        out_dir_level2 = session_dir + '\pds\level2'
    endif

    if n_elements(out_dir_level3) eq 0 then begin
        out_dir_level3 = session_dir + '\pds\level3'
    endif

    ; make sure that the output directory exists
    if file_test(out_dir_level2) eq 0 then begin
       print, 'Output Directory does not exists - Creating It: ' + out_dir_level2
       file_mkdir, out_dir_level2
    endif

    if file_test(out_dir_level3) eq 0 AND NOT keyword_set(SKIP_LEVEL3) then begin
       print, 'Output Directory does not exists - Creating It: ' + out_dir_level3
       file_mkdir, out_dir_level3
    endif

	if n_elements(JPEG_LEVEL2_DIR) ne 0 then begin
		if file_test(JPEG_LEVEL2_DIR) eq 0 then file_mkdir, JPEG_LEVEL2_DIR
	endif
	if n_elements(THUMB_LEVEL2_DIR) ne 0 then begin
		if file_test(THUMB_LEVEL2_DIR) eq 0 then file_mkdir, THUMB_LEVEL2_DIR
	endif
	if n_elements(JPEG_LEVEL3_DIR) ne 0 then begin
		if file_test(JPEG_LEVEL3_DIR) eq 0 then file_mkdir, JPEG_LEVEL3_DIR
	endif
	if n_elements(THUMB_LEVEL3_DIR) ne 0 then begin
		if file_test(THUMB_LEVEL3_DIR) eq 0 then file_mkdir, THUMB_LEVEL3_DIR
	endif

	print, '----------------------------------'
	print, '- Calibrating Directory '
    print, '- Source Directory:      ' + in_dir
    print, '- Level2 Directory:      ' + out_dir_level2
    if NOT keyword_set(SKIP_LEVEL3) then print, '- Level3 Directory:      ' + out_dir_level3

    ; find files to process
    mask = in_dir + '\*ID*'

	; handle the /NAC and /WAC keyword input
    if keyword_Set(NAC) then mask = in_dir + '\N*ID*.IMG'
    if keyword_Set(WAC) then mask = in_dir + '\W*ID*.IMG'


	; If FILTER is specified then override the previous mask
    if n_elements(FILTER) ne 0 then mask = mask + FILTER

	; find the input files
    files = findfile(mask)


    if n_elements(LOGFILE) eq 0 then begin
       unit = -1
    endif else begin
       openw, unit, LOGFILE, /GET_LUN
    endelse

    for i=0, n_elements(files)-1 do begin

       ok = 1

       ; generate output filename
       fd = p_parse_file_path(files[i])

       ; check compatible filetypes
       if strpos(fd.name, 'PA') gt 0 then continue
       if strpos(fd.name, 'PB') gt 0 then continue
       if strpos(fd.name, 'OL') gt 0 then continue
       if strupcase(strmid(fd.name,0,1)) eq 'N' then camera = 'NAC' else camera = 'WAC'
       if camera eq 'NAC' AND strpos(fd.name, '_F54') gt 0 then continue
       if camera eq 'WAC' AND strpos(fd.name, '_F87') gt 0 then continue


       ; rename output file for CODMAC level
       fid1 = strmid(fd.name, 0, 1)
       fid3 = strmid(fd.name, 0, 3)
       len = strlen(fd.filename)

       level2_file = out_dir_level2 + '\' + strmid(fd.name, 0, 31) + '2' + strmid(fd.name, 32) + '.IMG'
       level3_file = out_dir_level3 + '\' + strmid(fd.name, 0, 31) + '3' + strmid(fd.name, 32) + '.IMG'

       if keyword_set(PDS_ARCHIVE) then begin
            level2_file = out_dir_level2 + '\' + strupcase(strmid(fd.name, 0, 21) + '3' + strmid(fd.name, 22) + '.IMG')
            level3_file = out_dir_level3 + '\' + strupcase(strmid(fd.name, 0, 21) + '4' + strmid(fd.name, 22) + '.IMG')
       endif


       osiris_calibrate_file, files[i], level2_file, level3_file, REPROCESS=REPROCESS, LOGUNIT=unit, $
                          DELETE_HISTORY=DELETE_HISTORY, $
                          PDS_ARCHIVE=PDS_ARCHIVE, $
                          SKIP_LEVEL3=SKIP_LEVEL3, $
                          LEVEL2_IMAGE=d2, $
                          LEVEL3_IMAGE=d3, $
                          _EXTRA=_EXTRA

	  if n_elements(JPEG_LEVEL2_DIR) ne 0 AND n_elements(d2) ne 0 then begin
	    jfs = p_parse_file_path(level2_file)
	    file = JPEG_LEVEL2_DIR + '\' + jfs.name + '.jpg'
	  	write_jpeg, file, bytscl(fw_hist_stretch(d2)), QUALITY=80
	  endif

	  if n_elements(THUMB_LEVEL2_DIR) ne 0 AND n_elements(d2) ne 0 then begin
	    jfs = p_parse_file_path(level2_file)
	    file = THUMB_LEVEL2_DIR + '\' + jfs.name + '.jpg'
	    s = size(d2, /DIM)
	    mdim = s[0]
	    if s[1] gt mdim then mdim = s[1]
	    sf = float(mdim) / 128.0
	    tw = s[0] / sf
	    th = s[1] / sf
	    if tw le 1 then tw = 2
	    if th le 1 then th = 2
	    t2 = congrid(d2, tw, th, CUBIC=-0.5)
	  	write_jpeg, file, bytscl(fw_hist_stretch(t2)), QUALITY=80
	  endif

	  if n_elements(JPEG_LEVEL3_DIR) ne 0 AND n_elements(d3) ne 0 then begin
	    jfs = p_parse_file_path(level3_file)
	    file = JPEG_LEVEL3_DIR + '\' + jfs.name + '.jpg'
	  	write_jpeg, file, bytscl(fw_hist_stretch(d3)), QUALITY=80
	  endif

	  if n_elements(THUMB_LEVEL3_DIR) ne 0 AND n_elements(d3) ne 0 then begin
	    jfs = p_parse_file_path(level3_file)
	    file = THUMB_LEVEL3_DIR + '\' + jfs.name + '.jpg'
	    s = size(d3, /DIM)
	    mdim = s[0]
	    if s[1] gt mdim then mdim = s[1]
	    sf = float(mdim) / 128.0
	    tw = s[0] / sf
	    th = s[1] / sf
	    if tw le 1 then tw = 2
	    if th le 1 then th = 2
	    t3 = congrid(d3, tw, th, CUBIC=-0.5)
	  	write_jpeg, file, bytscl(fw_hist_stretch(t3)), QUALITY=80
	  endif
	endfor

    if unit gt 0 then free_lun, unit

END
