; Copyright (c) 2004-2010, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   FUNCTION osiris_can_calibrate, header
;
; PURPOSE:
;   Make a quick check to se if it at all makes sense to go ahead with the calibration
;
; CATEGORY:
;   Osiris CCD Calibration
;
; CALLING SEQUENCE:
;  Result = osiris_can_calibrate(header_in)
;
; INPUTS:
;   header_in:   input level 1 PDS header structure
;
; KEYWORD PARAMETERS:
;   NONE
;
; RETURNS:
;   0 on failure   1 on success
;
; RESTRICTIONS:
;   function requires that the paths defined in get_osiris_environment.pro has been set to correctly
;   point to the root directory of the external calibration files
;
; EXAMPLE:
;   p_read, pickfile(), h_in, d_in                 ; read in raw PDS image
;   if osiris_can_calibrate(h) ne 0 then begin
;   	osiris_level1_to_level2, h_in, d_in, h_out, d_out ; calibrate the image
;   	p_write, pickfile(), h_out, d_out             ; save the image to a new PDS file
;   endif
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid   3/11-2006
;   Modified by MK to handle filter number correctly for old images,  2008
;   Added keyword skip_flatfield to allow processing of images with no flat field available, 25 Jan 2009.
;-

FUNCTION osiris_can_calibrate, header, SKIP_FLATFIELD=SKIP_FLATFIELD

	; extract some header information
	osiris_model, header, camera, model
	fnumber = strtrim(p_value(header, 'SR_MECHANISM_STATUS.FILTER_NUMBER'), 2)
	if n_elements(fnumber) eq 2 then fnumber = fnumber[0] + fnumber[1] ;This line was added by MK

	; checl for valid flatfield
	if NOT KEYWORD_SET(SKIP_FLATFIELD) then begin     ;The if statement was added by MK
	   	flatfield_file = p_resolve_database_file('FLATFIELDS', camera, model, 'FLAT' + '-' + fnumber, '', 'IMG')
       	if flatfield_file eq '' then begin
       		printf, -2, 'WARNING - No flatfield found for F' + fnumber
       		return, 0	; flat fields not available
       	endif
    endif

	; Check for shutter error
	if p_value(header, 'SR_SHUTTER_STATUS.ERROR_TYPE_ID') ne 'NONE' then begin
		printf, -2, 'WARNING - Image with shutter error'
		return, 0
	endif

	; make sure that the image data is completely inside the CCD frame (So no pre/post/OL data alowed)
	x = p_value(header, 'IMAGE.FIRST_LINE_SAMPLE')-1
	y = p_value(header, 'IMAGE.FIRST_LINE')-1
	w = p_value(header, 'IMAGE.LINE_SAMPLES')
	h = p_value(header, 'IMAGE.LINES')

	bin = p_value(header, 'SR_ACQUIRE_OPTIONS.HARDWARE_BINNING_ID')
	swbin = (p_value(header, 'SR_COMPRESSION.PIXEL_AVERAGING_WIDTH'))[0]

	bf = 1
	if bin eq '2x2' then bf = 2
	if bin eq '4x4' then bf = 4
	if bin eq '8x8' then bf = 8

	if swbin gt 1 then bf = bf * swbin

	if x lt 0 OR x+w*bf gt 2048 OR y+h*bf gt 2048 then begin
		printf, -2, 'Image contains non CCD data - Not currently supported'
		return, 0
	endif

	; At this point we can calibrate the image
	return, 1

END